/* chaossim - a virtual machine to run CHAOS on, using curses.
 * Hopefully this should now load CP/M as well... maybe...
 * Adam Sampson, 1998.
 * 
 * Based heavily upon:
 * Z81, a VGA ZX81 emulator for Linux. (azz:which was based upon xz80 by..)
 * Copyright (C) 1994 Ian Collier. Z81 changes (C) 1995-6 Russell Marks.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <curses.h>
#include <getopt.h>
#include "z80.h"

int opt_logio=0, opt_nop=0, opt_logall=0;

unsigned char mem[65536],*helpscrn;
unsigned char *dfile;
unsigned char *iptr=mem;
int kports[8];
unsigned long tstates=0,tsmax=(1<<30);
int hires=0,hiscrn;
int help=0;
int topspeed=0,lowref=0;

/* I assume 0=ROM, 1=RAM, 2=video RAM? */
/* int memattr[4]={0,2,1,1}; */
unsigned char *memptr[4]={mem,mem+0x4000,mem+0x8000,mem+0xc000};
int memattr[4]={1,1,1,1};

int screen_dirty;
int hsize=256,vsize=192;
volatile int interrupted=0;
unsigned char *vptr;
int forceok=0;
int scrn_freq=2;
int booting=1;
int input_wait=0;

unsigned char chrmap_old[24*32],chrmap[24*32];
unsigned char himap_old[192*32],himap[192*32];

int refresh_screen=1;
unsigned int insult=ERR;

void sighandler(int a) {
   if(interrupted<2) interrupted=1;
}

void dontpanic(a) {
   endwin();
   printf("safe exit..\n");
   exit(1);
}

char *libdir(char *file) {
   static char buf[1024];
   sprintf(buf,"%s/%s",LIBDIR,file);
   return(buf);
}


/* main */

main(int argc, char **argv) {
 
  int c, i;

  /* Get the arguments. */

  while (1) {
    static struct option long_options[] = {
      {"image",1,0,0},
      {"nop",0,0,0},
      {"help",0,0,0},
      {"logio",0,0,0},
      {"logall",0,0,0},
      {0,0,0,0}
    };
    i = 0;
    c = getopt_long(argc, argv, "i:nhoa", long_options, &i);
    if (c==-1) break; /* out of the while */
 
    if (c==0) switch(i) {
    case 0: c = 'i'; break;
    case 1: c = 'n'; break;
    case 2: c = 'h'; break;
    case 3: c = 'o'; break;
    case 4: c = 'a'; break;
    };
   
    switch(c) {
    case 'h':
      printf("
z80em v2.0, various artists. Contact azz@josstix.demon.co.uk.

Options:
-h, --help             This help.
-i N, --image N        Select the memory image to load.
-a, --logall           Dump registers to stderr before each instruction.
-n, --nop              Dump registers to stderr on each NOP.
-o, --logio            Log INs and OUTs to stderr.

");
      exit(0);
      break;
    case 'i':
      loadrom(mem, optarg);
      break;
    case 'n':
      opt_nop = 1;
      break;
    case 'o':
      opt_logio = 1;
      break;
    case 'a':
      opt_logall = 1;
      break;
    }
  };

  initscr(); cbreak(); noecho(); nonl(); nodelay(stdscr, TRUE);
  intrflush(stdscr, FALSE); keypad(stdscr, TRUE);
  scrollok(stdscr, TRUE); idlok(stdscr, TRUE);
  
  mainloop();
  
  endwin();
  exit(0);
}

loadrom(unsigned char *x, char *name) {
   int i;
   FILE *in;
   
   if((in=fopen(name,"rb"))!=NULL)
     {
	for (i=0; !feof(in); i++)
	  x[i] = fgetc(in);
	fclose(in);
     }
   else
     {
	printf("z80em: couldn't load image %s.\n",name);
	exit(1);
     }
}

unsigned int in(int h,int l) {
   unsigned int j;

   if (opt_logio) fprintf(stderr, "in port (%02x)%02x\n", h, l);
   
   if (l==0x00) {
      if (insult == ERR) {
	 nodelay(stdscr, FALSE);
	 j = (unsigned int)getch();
	 return j;
      } else {	 
	 j = insult;
	 insult = ERR;
	 return j;
      };
   };
   if (l==0x01) {
      if (insult == ERR) {
	 nodelay(stdscr, TRUE);
	 insult = (unsigned int)getch();
	 if (insult == ERR) return 1; else return 0;
      } else {
	 return 1;
      };
   };
   
   if (opt_logio) fprintf(stderr,"in not handled: 0xFF returned.\n");
   return(255);
}

unsigned int out(int h,int l,int a) {

   if (opt_logio) fprintf(stderr, "out port (%02x)%02x value %02x\n",h,l,a);
   if ((l == 0x00) && (a != 0x0d)) {
      addch((chtype)a); 
      refresh();
   }
   
   return(0);
}


fix_tstates() {
   tstates=0;
   pause();
}

do_interrupt() {
   if(booting)
     if(input_wait)
       booting=0;
   else
     return(0);
   
   if(interrupted<2) interrupted=0;
}

